package com.sun.xml.internal.bind.v2.runtime;

import java.util.Iterator;
import javax.xml.namespace.QName;
import java.util.HashSet;
import java.util.HashMap;
import com.sun.xml.internal.bind.v2.util.QNameMap;
import java.util.Set;
import java.util.Map;

public final class NameBuilder
{
    private Map<String, Integer> uriIndexMap;
    private Set<String> nonDefaultableNsUris;
    private Map<String, Integer> localNameIndexMap;
    private QNameMap<Integer> elementQNameIndexMap;
    private QNameMap<Integer> attributeQNameIndexMap;
    
    public NameBuilder() {
        this.uriIndexMap = new HashMap();
        this.nonDefaultableNsUris = new HashSet();
        this.localNameIndexMap = new HashMap();
        this.elementQNameIndexMap = new QNameMap();
        this.attributeQNameIndexMap = new QNameMap();
    }
    
    public Name createElementName(QName name) {
        return this.createElementName(name.getNamespaceURI(), name.getLocalPart());
    }
    
    public Name createElementName(String nsUri, String localName) {
        return this.createName(nsUri, localName, false, this.elementQNameIndexMap);
    }
    
    public Name createAttributeName(QName name) {
        return this.createAttributeName(name.getNamespaceURI(), name.getLocalPart());
    }
    
    public Name createAttributeName(String nsUri, String localName) {
        assert nsUri.intern() == nsUri;
        assert localName.intern() == localName;
        if (nsUri.length() == 0) {
            return new Name(this.allocIndex(this.attributeQNameIndexMap, "", localName), -1, nsUri, this.allocIndex(this.localNameIndexMap, localName), localName, true);
        }
        this.nonDefaultableNsUris.add(nsUri);
        return this.createName(nsUri, localName, true, this.attributeQNameIndexMap);
    }
    
    private Name createName(String nsUri, String localName, boolean isAttribute, QNameMap<Integer> map) {
        assert nsUri.intern() == nsUri;
        assert localName.intern() == localName;
        return new Name(this.allocIndex(map, nsUri, localName), this.allocIndex(this.uriIndexMap, nsUri), nsUri, this.allocIndex(this.localNameIndexMap, localName), localName, isAttribute);
    }
    
    private int allocIndex(Map<String, Integer> map, String str) {
        Integer i = (Integer)map.get(str);
        if (i == null) {
            i = Integer.valueOf(map.size());
            map.put(str, i);
        }
        return (int)i;
    }
    
    private int allocIndex(QNameMap<Integer> map, String nsUri, String localName) {
        Integer i = (Integer)map.get(nsUri, localName);
        if (i == null) {
            i = Integer.valueOf(map.size());
            map.put(nsUri, localName, i);
        }
        return (int)i;
    }
    
    public NameList conclude() {
        boolean[] nsUriCannotBeDefaulted = new boolean[this.uriIndexMap.size()];
        for (Map.Entry<String, Integer> e : this.uriIndexMap.entrySet()) {
            nsUriCannotBeDefaulted[(int)e.getValue()] = this.nonDefaultableNsUris.contains(e.getKey());
        }
        NameList r = new NameList(this.list(this.uriIndexMap), nsUriCannotBeDefaulted, this.list(this.localNameIndexMap), this.elementQNameIndexMap.size(), this.attributeQNameIndexMap.size());
        this.uriIndexMap = null;
        this.localNameIndexMap = null;
        return r;
    }
    
    private String[] list(Map<String, Integer> map) {
        String[] r = new String[map.size()];
        for (Map.Entry<String, Integer> e : map.entrySet()) {
            r[(int)e.getValue()] = (String)e.getKey();
        }
        return r;
    }
}
